// Copyright (c) 2007 Helsinki Institute of Information Technology HIIT
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Gazillion
{
    /// <summary>
    /// Records a bag of fixations
    /// </summary>
    public class FixationBagRecorder : FixationRecorder
    {
        // times fixated at word n
        // Index [0] is reserved for 'someplace else'
        Dictionary<int, int> wordFixations;
        // msec fixated at word n
        // Index [0] is reserved for 'someplace else'
        Dictionary<int, int> wordTimes;

        // Thresholds for words to be considered relevant
        const int MSEC_THRESHOLD = 300;
        const int TIMES_THRESHOLD = 2;

        // Constructor
        public FixationBagRecorder(WebBrowser browser)
            : base(browser)
        {
            Clear();
        }

        // Record fixation in segmentTimes and wordTimes array
        override public void FixationEnd(int x, int y, int msec)
        {
            int word = getWindowWordId(x, y);
            if (word == 0)
                return;
            // word already exists
            if (!wordTimes.ContainsKey(word))
            {
                wordTimes.Add(word, msec);
                wordFixations.Add(word, 1);
            }
            else
            {
                wordTimes[word] += msec;
                wordFixations[word]++;
            }
        }

        // Reset fixation records
        override public void Clear()
        {
            this.wordTimes = new Dictionary<int, int>();
            this.wordFixations = new Dictionary<int, int>();
        }

        // Returns a bag of fixations for this trajectory
        public override FixationBag GetFixation()
        {
            FixationBag fixation = new FixationBag();
            foreach (int wordId in wordFixations.Keys)
            {
                int msec = wordTimes[wordId];
                int fixations = wordFixations[wordId];
                if (fixations > 0)
                {
                    string word = this.getWord(wordId);
                    if (word == "")
                        continue;
                    fixation.Add(word, msec, fixations);
                }
            }
            return fixation;
        }
        
        // Return words which are thought to be relevant
        override public ICollection<string> Relevant()
        {
            List<string> result = new List<string>();
            foreach (int wordId in wordFixations.Keys)
            {
                if (IsRelevant(wordId))
                {
                    string word = this.getWord(wordId);
                    if (word == "")
                        continue;
                    result.Add(word);
                }
            }
            return result;
        }

        // Show relevant words on browser
        public override void Show()
        {
            resetHighlights();
            foreach (int wordId in wordFixations.Keys)
            {
                if (IsRelevant(wordId))
                {
                    highlightWord(wordId);
                }
            }
        }

        // Test if wordId is relevant
        private bool IsRelevant(int wordId)
        {
            return (wordFixations[wordId] >= MSEC_THRESHOLD
                    || wordTimes[wordId] >= TIMES_THRESHOLD);
        }


    }

}