// Copyright (c) 2007 Helsinki Institute of Information Technology HIIT
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO; // FIXME For configuration file deletion

namespace Gazillion
{
    /// <summary>
    /// Browser main form. Maintains application window. 
    /// Contains webBrowser component and glue code to send events between 
    /// ETU-Driver and browser.
    /// </summary>
    public partial class GazillionForm : Form
    {

        private FixationBagRecorder history;
        private ETUDriver.CoETUDriverClass iETUDriver;
        private string ProfilePath { get { return Application.StartupPath + @"\"; } }
        private Search searchResult;
        private string searchTextBox_OldText = "";
        private int fixationEventsCaptured;

        public GazillionForm()
        {
            InitializeComponent();

            captureToolStripMenuItem.CheckState = CheckState.Unchecked;
            captureToolStripMenuItem.CheckStateChanged += captureItem_checkStateChanged;

            // Add callbacks to back and forward menu items
            webBrowser1.CanGoBackChanged += webBrowser1_CanGoBackChanged;
            webBrowser1.CanGoForwardChanged += webBrowser1_CanGoForwardChanged;
            //searchResult.ResultUpdated += search_SearchResultUpdate;

            // Initialize ETUDriver, part taken from ETUDriverClient example
            iETUDriver = new ETUDriver.CoETUDriverClass();
            // FIXME ETUDriver crashes is configuration file is present
            File.Delete(ProfilePath + Properties.Resources.ETUDriverINI);
            iETUDriver.OptionsFile = ProfilePath + Properties.Resources.ETUDriverINI;

            if (iETUDriver.Index < 0)
            {
                int size = 128;
                String buff = new String(' ', size);
                for (int i = 0; i < iETUDriver.DeviceCount; i++)
                {
                    iETUDriver.getNameFromIndex(i, ref buff, ref size);
                    string name = buff.Substring(0, size - 1);
                    if (name == "Mouse")
                    {
                        iETUDriver.Index = i;
                        break;
                    }
                }
            }

            // Add ETU-Driver event callbacks
            iETUDriver.OnCalibrated += new ETUDriver.IETUDriverEvents_OnCalibratedEventHandler(iETUDriver_OnCalibrated);
            iETUDriver.OnDataEvent += new ETUDriver.IETUDriverEvents_OnDataEventEventHandler(iETUDriver_OnDataEvent);
            iETUDriver.OnAppControlEvent += new ETUDriver.IETUDriverEvents_OnAppControlEventEventHandler(iETUDriver_OnAppControlEvent);

            // Update options- and calibrate UI elements
            optionsToolStripMenuItem.Enabled = (iETUDriver.Ready == 1);
            calibrateToolStripMenuItem.Enabled = (iETUDriver.Ready == 1);

            // Search result and search interface
            searchResult = new Search(webBrowser1);

            // Navigate to start page
            homeItem_Click(this, null);
        }




        /************************************************************************
         * ETU Driver Callbacks
         ***********************************************************************/

        // ETU-Driver calibration callback
        private void iETUDriver_OnCalibrated()
        {
            calibrateToolStripMenuItem.Enabled = (iETUDriver.Ready == 1);
            captureToolStripMenuItem.Enabled = (iETUDriver.Calibrated == 1);
            trackerStatusLabel.Text = "Stopped";
        }

        // ETU-Driver events
        private void iETUDriver_OnDataEvent(ETUDriver.EiETUDGazeEvent aEventID, ref int aData, ref int aResult)
        {
            // Ignore fixation if window is minimized or browser is navigating
            if (WindowState == FormWindowState.Minimized
                || history == null)
                return;


            Rectangle rect = webBrowser1.DisplayRectangle;

            switch (aEventID)
            {
                case ETUDriver.EiETUDGazeEvent.geFixationStart:
                    {
                        ETUDriver.SiETUDFixation fix = new ETUDriver.SiETUDFixation();
                        fix = iETUDriver.LastFixation;
                        // Ignore events outside of browser window
                        if (fix.X >= rect.X && fix.X <= fix.X + rect.Width
                        && fix.Y >= rect.Y && fix.Y <= fix.Y + rect.Height)
                            history.FixationStart((int)fix.X, (int)fix.Y, (int)fix.Duration);
                        break;
                    }

                case ETUDriver.EiETUDGazeEvent.geFixationUpdate:
                    {
                        ETUDriver.SiETUDFixation fix = new ETUDriver.SiETUDFixation();
                        fix = iETUDriver.LastFixation;
                        // Ignore events outside of browser window
                        if (fix.X >= rect.X && fix.X <= fix.X + rect.Width
                        && fix.Y >= rect.Y && fix.Y <= fix.Y + rect.Height)
                            history.FixationUpdate((int)fix.X, (int)fix.Y, (int)fix.Duration);
                        break;
                    }
                case ETUDriver.EiETUDGazeEvent.geFixationEnd:
                    {
                        ETUDriver.SiETUDFixation fix = new ETUDriver.SiETUDFixation();
                        fix = iETUDriver.LastFixation;
                        // Ignore events outside of browser window
                        if (fix.X >= rect.X && fix.X <= fix.X + rect.Width
                        && fix.Y >= rect.Y && fix.Y <= fix.Y + rect.Height)
                        {
                            history.FixationEnd((int)fix.X, (int)fix.Y, (int)fix.Duration);
                            fixationEventCaptured();
                        }
                        break;
                    }

            }

            aResult = 1;
        }

        // ETU-Driver application event
        private void iETUDriver_OnAppControlEvent(ETUDriver.EiETUDAppControlMessage
            aMessageID, ref int aData, ref int aResult)
        {
            switch (aMessageID)
            {
                case ETUDriver.EiETUDAppControlMessage.acmToggleFullScreen:
                    {
                        if (WindowState == FormWindowState.Minimized)
                            break;

                        switch (aData)
                        {
                            case 0:  // maximize
                                if (WindowState != FormWindowState.Normal)
                                    WindowState = FormWindowState.Normal;
                                break;

                            case 1:  // back to normal
                                if (WindowState != FormWindowState.Maximized)
                                    WindowState = FormWindowState.Maximized;
                                break;

                            case 2:  // toggle the window state
                                WindowState = (WindowState == FormWindowState.Normal
                                    ? FormWindowState.Maximized
                                    : FormWindowState.Normal);
                                break;
                        }
                    }
                    break;

                case ETUDriver.EiETUDAppControlMessage.acmKeyPress:
                    break;

                case ETUDriver.EiETUDAppControlMessage.acmGetProfileName:
                    {
                        aResult = 0;
                        aData = 0;
                        break;
                    }

                case ETUDriver.EiETUDAppControlMessage.acmGetProfileDir:
                    {
                        aResult = 0;
                        aData = 0;
                        // aResult = ProfilePath.Length;
                        // char* target = (char*)aData;
                        // I didn't find how copy dir to aData. 
                        // Tobii Eye Tracker wants to know where to save calibration data,
                        // thus aData must receive this string somehow;
                        break;
                    }
            }
        }


        /************************************************************************
         * UI Callbacks
         ***********************************************************************/

        // Eye movements tracking changed, pass to ETU-Driver
        private void captureItem_checkStateChanged(object sender, EventArgs e)
        {
            if (iETUDriver.Active == 1)
            {
                trackerStatusLabel.Text = "Stopped";
                iETUDriver.stopTracking();
            }
            else
            {
                trackerStatusLabel.Text = "Tracking";
                iETUDriver.startTracking();
            }

            optionsToolStripMenuItem.Enabled = (iETUDriver.Active == 0);
            calibrateToolStripMenuItem.Enabled = (iETUDriver.Active == 0);
            showRelevantItem.Enabled = (iETUDriver.Active == 1);
            suggestToolStripMenuItem.Enabled = (iETUDriver.Active == 1);
        }

        // Show ETU-Driver options dialog
        private void optionsItem_Click(object sender, EventArgs e)
        {
            iETUDriver.showRecordingOptions();
            calibrateToolStripMenuItem.Enabled = (iETUDriver.Ready == 1);
        }

        // Show page or restore text to document title on location text
        private void locationText_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Jump to page on Enter key
            if (e.KeyChar == (char)Keys.Return)
            {
                e.Handled = true;
                navigate(locationText.Text);
                return;
            }
            // Restore on Escape key
            if (e.KeyChar == (char)Keys.Escape)
            {
                locationText.Text = webBrowser1.Document.Title;
                e.Handled = true;
            }

        }

        // Enter location text
        private void locationItem_Click(object sender, EventArgs e)
        {
            locationText.Focus();
        }

        // Restore location text to document title
        private void locationText_Leave(object sender, EventArgs e)
        {
            if (webBrowser1.Document != null)
                locationText.Text = webBrowser1.Document.Title;
        }

        // Enter search text
        private void wikiSearchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            searchTextBox_OldText = searchTextBox.Text;
            searchTextBox.Text = "";
            searchTextBox.Focus();
        }

        // Suggest documents
        private void suggestToolStripMenuItem_Click(object sender, EventArgs e)
        {
            suggest();
        }

        // Show help page on browser
        private void releaseNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            help();
        }

        // Open about dialog
        private void aboutItem_Click(object sender, EventArgs e)
        {
            AboutDialog about = new AboutDialog();
            about.Show();
        }

        // User clicked browser 'back' button
        private void backItem_click(object sender, EventArgs e)
        {
            webBrowser1.GoBack();
        }

        // User clicked browser 'forward' button
        private void forwardItem_Click(object sender, EventArgs e)
        {
            webBrowser1.GoForward();
        }

        // User clicked browser 'home' button
        private void homeItem_Click(object sender, EventArgs e)
        {
            webBrowser1.Navigate(Properties.Settings.Default.urlHome);

        }

        // Quit application
        private void quitItem_Click(object sender, EventArgs e)
        {
            iETUDriver.stopTracking();
            this.Close();
        }

        // Calibrate eye-tracker
        private void calibrateItem_Click(object sender, EventArgs e)
        {
            iETUDriver.calibrate();
        }

        // Show ETU-Driver about dialog
        private void aboutETUDriverItem_Click(object sender, EventArgs e)
        {
            int size = 128;
            string buff = new string(' ', size);
            iETUDriver.getVersion(1, ref buff, ref size);

        }

        // Highlight relevant words from history on browser
        private void showRelevantItem_Click(object sender, EventArgs e)
        {
            history.Show();
        }

        // Refresh search results per eye tracking history
        private void refreshItem_Click(object sender, EventArgs e)
        {
            if (this.searchResult != null)
            {
                this.searchResult.ChangeCues(history);
            }

        }

        // Update search results list
        private void SearchResultUpdate(ICollection<SearchResult> result)
        {
            searchListBox.Items.Clear();
            foreach (SearchResult match in result)
            {
                searchListBox.Items.Add(match.Title);
            }
        }

        // Show user clicked search result
        private void searchListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Navigate to selected page and deselect item
            if (searchListBox.SelectedItems.Count != 0)
            {
                navigate((string)searchListBox.SelectedItem);
                searchListBox.SetSelected(searchListBox.SelectedIndex, false);
            }
        }

        // Run search query or restore text
        private void searchTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Run search query on Enter key
            if (e.KeyChar == (char)Keys.Return)
            {
                e.Handled = true;
                search(searchTextBox.Text);
                return;
            }
            // Restore old search text on Escape key
            if (e.KeyChar == (char)Keys.Escape)
            {
                e.Handled = true;
                searchTextBox.Text = searchTextBox_OldText;
                return;
            }
        }

        // Keep old search text for restoring in case user escapes
        private void searchTextBox_Enter(object sender, EventArgs e)
        {
            searchTextBox_OldText = searchTextBox.Text;
        }

        // Clear search text, search results and eyetracking history, update UI
        private void clearToolStripButton_Click(object sender, EventArgs e)
        {
            history = new FixationBagRecorder(webBrowser1);
            // Clear search text, earch results
            searchResult.Clear();
            searchListBox.Items.Clear();
            // Disable UI buttons
            this.clearToolStripButton.Enabled = false;
            this.suggestButton.Enabled = false;
            this.refreshResultsButton.Enabled = false;
            // Update fixation counter on UI
            fixationEventsCaptured = 0;
            this.fixationCountLabel.Text = "" + this.fixationEventsCaptured;
        }

        // Increase fixation events counter and update UI
        private void fixationEventCaptured()
        {
            if (this.fixationEventsCaptured == 0)
            {
                // Update 'Clear' button
                this.clearToolStripButton.Enabled = true;
                // Update 'Suggest' button
                this.suggestButton.Enabled = true;
                // Update 'Refresh search' button
                if (!this.searchResult.Empty() && this.searchResult.HasFreetext())
                    this.refreshResultsButton.Enabled = true;
            }
            // Update fixation counter on UI
            this.fixationEventsCaptured++;
            this.fixationCountLabel.Text = "" + this.fixationEventsCaptured;
        }

        private void preferencesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SettingsDialog settings = new SettingsDialog();
            settings.Show();

        }

        /************************************************************************
         * Browser callbacks
         ***********************************************************************/

        private void webBrowser1_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            history = new FixationBagRecorder(webBrowser1);
            // Document title
            string text = webBrowser1.Document.Title;
            text = text.Split(new string[] { " - " }, System.StringSplitOptions.None)[0];
            locationText.Text = text;
            // Update search results
            if (searchResult.HasSearchResults())
            {
                searchResult.UpdateResults();
                SearchResultUpdate(searchResult.Result());
            }
            webBrowser1.Focus();
        }

        private void webBrowser1_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            // Disable Suggest and Refresh Results buttons
            this.suggestButton.Enabled = false;
            this.refreshResultsButton.Enabled = false;
            // Reset captured trajectory, counters and highlighted words
            fixationEventsCaptured = 0;
            this.fixationCountLabel.Text = "0";
            history.resetHighlights();
            history = null;
        }

        // Change back menu items
        private void webBrowser1_CanGoBackChanged(object sender, EventArgs e)
        {
            backToolStripMenuItem.Enabled = webBrowser1.CanGoBack;
            backButton.Enabled = webBrowser1.CanGoBack;

        }

        // Change forward menu items
        private void webBrowser1_CanGoForwardChanged(object sender, EventArgs e)
        {
            forwardToolStripMenuItem.Enabled = webBrowser1.CanGoForward;
            forwardButton.Enabled = webBrowser1.CanGoForward;
        }


        /************************************************************************
         * Search, suggest and display cues
         ***********************************************************************/

        // Suggest relevant documents based on eye tracking history
        private void suggest()
        {
            searchResult.Suggest(history);
        }

        // Search wikipedia
        private void search(string needle)
        {
            searchResult.Freetext(needle);
        }

        // Display help page
        private void help()
        {
            webBrowser1.Navigate(Properties.Settings.Default.urlHelp);
        }

        // Navigate to page user entered
        private void navigate(string s)
        {
            // Page title
            string page = s.Replace(' ', '_');
            // TODO Suggest CamelCased text
            webBrowser1.Navigate(Properties.Settings.Default.urlPrefix + page);
        }


    }
}