// Copyright (c) 2007 Helsinki Institute of Information Technology HIIT
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Gazillion
{
    /// <summary>
    /// Interfaces with webBrowser control instance to provide search, read
    /// search results from page and store search results.
    /// </summary>
    class Search
    {
        // Freetext search string
        string freetext;
        // Fixations
        FixationBag fixations;
        // Results vector
        ICollection<SearchResult> result;
        // webBrowser control instance
        WebBrowser browser;

        public Search(WebBrowser browser)
        {
            this.browser = browser;
            Clear();
        }

        public void Clear()
        {
            this.freetext = "";
            this.result = new List<SearchResult>();
            this.fixations = new FixationBag();

        }

        public void Freetext(string freetext)
        {
            this.freetext = freetext;
            this.result = new List<SearchResult>();
            this.fixations = new FixationBag();
            Refresh();
        }

        public void Suggest(FixationRecorder history)
        {
            this.freetext = "";
            this.result = new List<SearchResult>();
            this.fixations = history.GetFixation();
            Refresh();
        }

        public void ChangeFreetext(string freetext)
        {
            this.freetext = freetext;
            Refresh();
        }

        public void ChangeCues(FixationRecorder history)
        {
            this.fixations = history.GetFixation();
            Refresh();
        }

        // Peek if page is a search results page
        public bool HasSearchResults()
        {
            return (this.getResultCount() > 0);
        }

        // Test if page has search results
        public bool Empty()
        {
            return (this.result.Count == 0);
        }

        // Test if the search has freetext string
        public bool HasFreetext()
        {
            return (this.freetext.Length == 0);
        }

        // Read results from page Javascript and populate result vector
        public void UpdateResults()
        {
            int results = getResultCount();
            for (int i = 0; i < results; ++i)
            {
                result.Add(getResult(i));
            }
        }

        // Returns search results vector
        public ICollection<SearchResult> Result()
        {
            return this.result;
        }

        // Access seach results from server 
        private void Refresh()
        {
            this.result = new List<SearchResult>();
            StringBuilder sb = new StringBuilder();
            sb.Append(freetext);
            if (this.fixations.Count() > 0)
                for (int i = 0; i < this.fixations.Count(); ++i)
                {
                    sb.Append("&").Append(this.fixations.GetWord(i)).Append("=")
                        .Append(this.fixations.GetFixations(i)).Append(",")
                        .Append(this.fixations.GetMsec(i));
                }
            browser.Navigate(Properties.Settings.Default.urlSearch + sb.ToString());
        }

        // Retrieve count of results from browser
        protected int getResultCount()
        {
            int count = Convert.ToInt32(browser.Document.InvokeScript("resultCount"));
            return count;
        }

        // Retrieve result from browser
        protected SearchResult getResult(int resultId)
        {
            if (resultId < 0 || resultId >= getResultCount())
                throw new Exception("SearchResult resultId not in range");
            string page = (string)browser.Document.InvokeScript("resultPage",
                new string[] { "" + resultId });
            string title = (string)browser.Document.InvokeScript("resultTitle",
                new string[] { "" + resultId });
            string description = (string)browser.Document.InvokeScript("resultDescription",
                new string[] { "" + resultId });
            //Console.Out.WriteLine("page='" + page + "' title='" + title + "'");
            /// TODO Exception type
            if (page == null || page == ""
                || title == null || title == ""
                || description == null)
                throw new Exception("SearchResult not found in page");
            return new SearchResult(title, page, description);
        }


    }
}
